@@ -4,6 +4,7 @@ import shutil
 import argparse 
 import textwrap
 import json
+import cdb_query_archive
 
 class Open_With_Indent:
     """This class creates an open file were indentation is tracked"""
@@ -34,16 +35,17 @@ class Experiment_Setup:
         if 'years' in dir(self):
             self.years=self.years.split(',')
 
+        self.runscript_file=self.runscripts_dir+'/'+'_'.join([self.diagnostic,
+                                                            self.model,
+                                                            self.center,
+                                                            self.rip,
+                                                            self.experiment,
+                                                            '-'.join(self.years)])
+
     def prepare_scripts(self):
         """ Prepares the scripts for bash launch """
     
-        runscript_file=self.runscripts_dir+'/'+'_'.join([self.diagnostic,
-                                                            self.model,
-                                                            self.center,
-                                                            self.run_id,
-                                                            self.experiment,
-                                                            '-'.join(self.years)])
-        out=Open_With_Indent(runscript_file,'w')
+        out=Open_With_Indent(self.runscript_file,'w')
 
         out.writei('#!/bin/bash\n')
         if self.pbs_expt:
@@ -51,9 +53,9 @@ class Experiment_Setup:
 	    if self.queue != None: out.writei('#PBS -q {0}\n'.format(self.queue))
             #out.writei('#PBS -l nodes=1:ppn={0},walltime={1}\n'.format(max(self.dim_async,self.m_async),self.walltime))
             out.writei('#PBS -l nodes=1:ppn={0}\n'.format(max(self.dim_async,self.m_async)))
-            out.writei('#PBS -N {0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment))
-            out.writei('#PBS -o {5}/pbs_out/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment,self.output_dir))
-            out.writei('#PBS -e {5}/pbs_err/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.run_id,self.experiment,self.output_dir))
+            out.writei('#PBS -N {0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.rip,self.experiment))
+            out.writei('#PBS -o {5}/pbs_out/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.rip,self.experiment,self.output_dir))
+            out.writei('#PBS -e {5}/pbs_err/{0}_{1}_{2}_{3}_{4}\n'.format(self.years[0],self.years[1],self.model,self.rip,self.experiment,self.output_dir))
 
         #Put the header to the diagnostic:
         out.writei('\n')
@@ -66,7 +68,7 @@ class Experiment_Setup:
         out.writei('CDB_YEARS="{0},{1}"\n'.format(self.years[0],self.years[1]))
         out.writei('export CDB_MODEL="{0}"\n'.format(self.model))
         out.writei('export CDB_CENTER="{0}"\n'.format(self.center))
-        out.writei('export CDB_RUN_ID="{0}"\n'.format(self.run_id))
+        out.writei('export CDB_RUN_ID="{0}"\n'.format(self.rip))
         out.writei('export CDB_EXPT="{0}"\n'.format(self.experiment))
         out.writei('export CDB_DIAG_NAME="{0}"\n'.format(self.diagnostic))
         out.writei('export CDB_DIAG_HEADER="{0}"\n'.format(os.path.abspath(self.diag_header)))
@@ -140,7 +142,7 @@ def structure_out_with_cmip5_drs(self,out):
     out.writei('\n')
     out.writei('for CDB_VAR_NAME in $CDB_VAR_LIST; do\n')
     out.inc_indent()
-    out.writei('CDB_LOCAL_RUN_ID={0}\n'.format(self.run_id))
+    out.writei('CDB_LOCAL_RUN_ID={0}\n'.format(self.rip))
     out.writei('if [ "$CDB_FREQ" == "fx" ]; then\n')
     out.inc_indent()
     out.writei('if [ "$CDB_MIP" == "fx" ]; then\n')
@@ -157,6 +159,8 @@ def structure_out_with_cmip5_drs(self,out):
     out.inc_indent()
     out.writei('fx) FILE_NAME="${FILE_NAME}";;\n')
     out.writei('clim) FILE_NAME="${FILE_NAME}";;\n')
+    out.writei('3hr) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}01010000-${CDB_YEAR_END}12312100";;\n')
+    out.writei('6hr) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}01010000-${CDB_YEAR_END}12311800";;\n')
     out.writei('day) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}0101-${CDB_YEAR_END}1231";;\n')
     out.writei('mon) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}01-${CDB_YEAR_END}12";;\n')
     out.writei('yr) FILE_NAME="${FILE_NAME}_${CDB_YEAR_START}-${CDB_YEAR_END}";;\n')
@@ -211,20 +215,19 @@ def start_monthly_loop(self,out,line):
     out.writei('#Retrieve path names to data:\n')
     for var_name in self.variable_list.keys():
         if self.variable_list[var_name][0]!='fx':
-            retrieval_string='${CDB_CENTER} ${CDB_MODEL} ${CDB_EXPT} ${CDB_RUN_ID} '
+            retrieval_string='--center=${CDB_CENTER} --model=${CDB_MODEL} --experiment=${CDB_EXPT} --rip=${CDB_RUN_ID} '
         else:
-            retrieval_string='${CDB_CENTER} ${CDB_MODEL} ${CDB_EXPT} r0i0p0 '
-        retrieval_string+=var_name+' '
-        retrieval_string+=' '.join(self.variable_list[var_name])
-        retrieval_string+=' ${CDB_YEAR} ${CDB_MONTH}'
+            retrieval_string='--center=${CDB_CENTER} --model=${CDB_MODEL} --experiment=${CDB_EXPT} --rip=r0i0p0 '
+        retrieval_string+='--var='+var_name+' '
+        retrieval_string+=' '.join(['='.join(item) for item in zip(['--frequency','--realm','--mip'],self.variable_list[var_name])])
+        retrieval_string+=' --year=${CDB_YEAR} --month=${CDB_MONTH}'
         retrieval_string+=' ${CDB_DIAG_HEADER}'
-        out.writei('export CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'=$(cdb_query_archive retrieve '+retrieval_string+')\n')
-        out.writei('export CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'_TYPE=$(cdb_query_archive retrieve -f '+retrieval_string+')\n')
+        out.writei('export CDB_'+var_name+'_'+'_'.join(self.variable_list[var_name])+'=$(cdb_query_archive list_paths '+retrieval_string+')\n')
     out.writei('\n')
 
     #Next we generate a script line that can be used in CDO:
     out.writei('#Create a cdo retrieval script:\n')
-    out.writei('CDO_RETRIEVAL_SCRIPT=""\n')
+    out.writei('export CDO_RETRIEVAL_SCRIPT=""\n')
     out.writei('if [ -z "$CDB_TIME_OFFSET" ]; then CDB_TIME_OFFSET=0; fi\n')
     for var_name in self.variable_list.keys():
         if not self.variable_list[var_name][0] in ['fx','clim']:
@@ -333,7 +336,7 @@ def main():
                             be used with --dim_async. Requires NCO version 4.0.0 and above, gnu-parallel.")
     proc_group.add_argument("-P","--pbs_expt",dest="pbs_expt",
                       default=False, action="store_true",
-                      help="Prepare a pbs header for each model/run_id/experiment")
+                      help="Prepare a pbs header for each model/rip/experiment")
     proc_group.add_argument("-w","--walltime",dest="walltime",
                       default="1:00:00",
                       help="pbs job walltime. Inactive for now: Determined by queue")
@@ -344,6 +347,26 @@ def main():
                       default=False, action="store_true",
                       help="Submit scripts to the PBS queue")
 
+    slicing_args={
+                  'center': [str,'Modelling center name'],
+                  'model': [str,'Model name'],
+                  'experiment': [str,'Experiment name'],
+                  'rip': [str,'RIP identifier, e.g. r1i1p1'],
+                  'var': [str,'Variable name, e.g. tas'],
+                  'frequency': [str,'Frequency, e.g. day'],
+                  'realm': [str,'Realm, e.g. atmos'],
+                  'mip': [str,'MIP table name, e.g. day'],
+                  'year': [int,'Year'],
+                  'month': [int,'Month as an integer ranging from 1 to 12'],
+                  'file_type': [str,'File type: OPEnDAP, local_file, HTTPServer, GridFTP']
+                  }
+
+    #Slicing options
+    slicing_group=parser.add_argument_group("Slicing",
+                            "Use these options to restrict the processing to a subset of models")
+    for arg in slicing_args.keys():
+        slicing_group.add_argument('--'+arg,type=slicing_args[arg][0],help=slicing_args[arg][1])
+
     options = parser.parse_args()
 
     #Load diagnostic description file:
@@ -378,11 +401,12 @@ def main():
         options.months_list=range(1,13)
 
     for simulation in paths_dict['simulations_list']:
-        options.center, options.model, options.run_id = simulation.split('_')
-        for experiment in diag_desc['experiment_list'].keys():
-            options.experiment=experiment
-            period_list=diag_desc['experiment_list'][experiment]
+        options.center, options.model, options.rip = simulation.split('_')
+        for exp in diag_desc['experiment_list'].keys():
+            options.experiment=exp
+            period_list=diag_desc['experiment_list'][exp]
             if not isinstance(period_list,list): period_list=[period_list]
+
             for period in period_list:
                 options.years=period
 
@@ -391,9 +415,9 @@ def main():
                 experiment.prepare_scripts()
 
                 if options.run:
-                    print 'Not implemented yet'
+                    os.system('bash '+experiment.runscript_file)
                 elif options.submit:
-                    print 'Not implemented yet'
+                    os.system('qsub '+experiment.runscript_file)
 
     if not options.run and not options.submit:
         print 'Scripts are available in '+options.runscripts_dir
