@@ -1,4 +1,28 @@
 import os
+#import md5
+import hashlib
+
+def open_json(options):
+    import json
+    if options.in_diagnostic_headers_file[-3:]=='.gz':
+        import gzip
+        infile=gzip.open(options.in_diagnostic_headers_file,'r')
+    else:
+        infile=open(options.in_diagnostic_headers_file,'r')
+    paths_dict=json.load(infile)
+    return paths_dict
+
+def close_json(paths_dict,options):
+    import json
+    if options.gzip:
+        import gzip
+        outfile = gzip.open(options.out_paths_file+'.gz','w')
+    else:
+        outfile = open(options.out_paths_file,'w')
+
+    json.dump(paths_dict,outfile)
+    outfile.close()
+    return
 
 def union_headers(diag_headers):
     #Create the diagnostic description dictionary:
@@ -24,16 +48,16 @@ def union_headers(diag_headers):
     return diag_desc
 
 #def find_optimset(diagnostic_headers_file, diagnostic_paths_file):
-def find_optimset(options):
+def find_optimset(in_paths_dict,options):
     import valid_experiments_path
-    import json
+
+    #Attribute the loaded path description:
+    paths_dict={}
+    paths_dict['diagnostic']=in_paths_dict
+
     #Tree description following the CMIP5 DRS:
     diag_tree_desc=['search','center','model','experiment','frequency','realm','mip','rip',
                     'version','var']
-
-    #Load the diagnostic header file:
-    paths_dict={}
-    paths_dict['diagnostic']=json.load(options.in_diagnostic_headers_file)
     
     #Create the diagnostic description dictionary with the union of all the requirements.
     #This will make the description easier and allow to appply the intersection of requirements
@@ -68,41 +92,7 @@ def find_optimset(options):
                          ]
     paths_dict=valid_experiments_path.intersection(paths_dict,diag_tree_desc,diag_tree_desc_path)
 
-    #Write dictionary to file using JSON encoding.
-    outfile = open(options.out_paths_file,'w')
-    json.dump(paths_dict,outfile)
-
-    outfile.close()
-
-def find_optimset_months(options):
-    import json
-    import valid_experiments_months
-    paths_dict=json.load(options.in_diagnostic_headers_file)
-
-    #Open the output file.If the gzip flag was used,
-    #output directly to a compressed file:
-    if options.gzip:
-        import gzip
-        outfile = gzip.open(options.out_paths_file+'.gz','w')
-    else:
-        outfile = open(options.out_paths_file,'w')
-
-    diag_tree_desc_path=[
-                            'center','model','experiment','rip','frequency','realm','mip',
-                            'var','version','search','file_type'
-                         ]
-    #Find the list of center / model with all the months for all the years / experiments and variables requested:
-    diag_tree_desc_months=[
-                            'experiment','center','model','rip','frequency','realm','mip',
-                            'var','year','month','file_type','version','search'
-                         ]
-    paths_dict=valid_experiments_months.intersection(paths_dict,diag_tree_desc_path,diag_tree_desc_months)
-
-    #Write dictionary to file using JSON encoding.
-    json.dump(paths_dict,outfile)
-
-    outfile.close()
-    
+    return paths_dict
 
 def search_filesystem(diag_desc,diag_tree_desc,top_path):
     import filesystem_query
@@ -112,100 +102,127 @@ def search_esgf(search_path,file_type_list,variable_list,experiment_list,diag_tr
     import esgf_query
     return esgf_query.descend_tree(search_path,file_type_list,variable_list,experiment_list,diag_tree_desc)
 
-def retrieval(retrieval_desc):
-    import json
-    #Reads the file created by find_optimset and gives the path to file:
-    if retrieval_desc.in_diagnostic_pointers_file[-3:]=='.gz':
-        import gzip
-        infile=gzip.open(retrieval_desc.in_diagnostic_pointers_file,'r')
-    else:
-        infile=open(retrieval_desc.in_diagnostic_pointers_file,'r')
-    paths_dict=json.load(infile)
+def find_optimset_months(paths_dict,options):
+    import valid_experiments_months
+    import database_utils
+    diag_tree_desc_path=[
+                            'center','model','experiment','rip','frequency','realm','mip',
+                            'var','version','search','file_type'
+                         ]
+    #Find the list of center / model with all the months for all the years / experiments and variables requested:
+    diag_tree_desc_months=[
+                            'experiment','center','model','rip','frequency','realm','mip',
+                            'var','year','month','version','file_type','search'
+                         ]
+    paths_dict=valid_experiments_months.intersection(paths_dict,diag_tree_desc_path,diag_tree_desc_months)
 
-    #Go down the tree and retrieve requested fields:
-    path_name=tree_retrieval(retrieval_desc,paths_dict['diagnostic'],paths_dict['data_pointers'])
-    if retrieval_desc.file_type_flag:
-        print path_name
-    else:
-        print path_name
+    #Find the unique tree:
+    paths_dict=database_utils.unique_tree(paths_dict,paths_dict['diagnostic'])
+    return paths_dict
+
+def list_paths(paths_dict,options):
+    list_of_paths=list_unique_paths(paths_dict,options)
+    if not list_of_paths is None:
+        for path in list_of_paths:
+            print path
     return
 
-def tree_retrieval(retrieval_desc,diag_desc,paths_dict):
+def list_unique_paths(paths_dict,options):
+    sliced_paths_dict=slice_data(paths_dict,options)
+    
+    #Go down the tree and retrieve requested fields:
+    if sliced_paths_dict['data_pointers']==None: return
+
+    path_names=tree_retrieval(sliced_paths_dict['data_pointers'],options)
+    unique_paths=[]
+    for path in sorted(set(path_names)):
+        if 'wget' in dir(options) and options.wget:
+            unique_paths.append('\''+'/'.join(path.split('|')[0].split('/')[-10:])+'\' \''+path.split('|')[0]+'\' \'MD5\' \''+path.split('|')[1]+'\'')
+        else:
+            unique_paths.append(path)
+    return unique_paths
+
+def tree_retrieval(paths_dict,options):
+    path_names=[]
     #Reads the tree recursively:
     if isinstance(paths_dict,dict):
         #Read the level name:
         level_name=paths_dict['_name']
-        if level_name in dir(retrieval_desc):
-            #If the level name was specified in the retrieval, use this specification:
-            path_name=tree_retrieval(retrieval_desc,diag_desc,paths_dict[str(getattr(retrieval_desc,level_name))])
-        elif level_name=='version':
-            #the 'version' field is peculiar. Here, we use the most recent, or largest version number:
-            version_list=[]
-            for version in paths_dict.keys():
-                if version[0]!='_':
-                    version_list.append(int(version[1:]))
-            version='v'+str(max(version_list))
-            path_name=tree_retrieval(retrieval_desc,diag_desc,paths_dict[version])
-        else:
-            #The level was not specified but an ordered list was provided in the diagnostic header.
-            #Go through the list and pick the first avilable one:
-            level_ordering=diag_desc[level_name+'_list']
-            for level in level_ordering:
-                if level in paths_dict.keys():
-                    if level_name=='file_type' and retrieval_desc.file_type_flag:
-                        #if the file_type_flag was used, the script outputs only the file_type
-                        return level
-                    path_name=tree_retrieval(retrieval_desc,diag_desc,paths_dict[level])
+        for level in paths_dict.keys():
+            if level[0]!='_':
+                setattr(options,level_name,level)
+                path_names.extend(tree_retrieval(paths_dict[level],options))
+    else:
+        path_names=[paths_dict[0]]
+    return path_names
+
+def find_local(paths_dict,options):
+    import database_utils
+
+    #In the search paths, find which one are local:
+    local_search_path=[ top_path for top_path in 
+                            [os.path.expanduser(os.path.expandvars(search_path)) for search_path in paths_dict['diagnostic']['search_list']]
+                            if os.path.exists(top_path)]
+    local_search_path=[ os.path.abspath(top_path) for top_path in local_search_path]
+
+    path_names=list_unique_paths(paths_dict,options)
+    path_equivalence=dict()
+    for path in path_names:
+        path_equivalence[path]=None
+        for search_path in local_search_path:
+            md5checksum=path.split('|')[1]
+            path_to_file=search_path+'/'+'/'.join(path.split('|')[0].split('/')[-10:])
+            try:
+                f=open(path_to_file)
+                if md5_for_file(f)==md5checksum:
+                    path_equivalence[path]=path_to_file
                     break
-    else:
-        path_name=paths_dict[0]
-    return path_name
+            except:
+                pass
 
-def simulations(simulations_desc):
-    import json
-    #Reads the file created by find_optimset and gives the path to file:
-    if simulations_desc.in_diagnostic_pointers_file[-3:]=='.gz':
-        import gzip
-        infile=gzip.open(simulations_desc.in_diagnostic_pointers_file,'r')
-    else:
-        infile=open(simulations_desc.in_diagnostic_pointers_file,'r')
-    paths_dict=json.load(infile)
+    restricted_paths_dict=paths_dict
+    restricted_paths_dict['data_pointers'], options=database_utils.replace_path(paths_dict['data_pointers'],options,path_equivalence)
+    return restricted_paths_dict
+                
+def md5_for_file(f, block_size=2**20):
+    md5 = hashlib.md5()
+    while True:
+        data = f.read(block_size)
+        if not data:
+            break
+        md5.update(data)
+    return md5.hexdigest()
 
+def simulations(paths_dict,options):
     for item in paths_dict['simulations_list']:
         print item
     return
 
-def slice_data(options):
-    import json
+def slice_data(paths_dict,options):
     import database_utils
     import copy
-    #Reads the file created by find_optimset and gives the path to file:
-    if options.in_diagnostic_pointers_file[-3:]=='.gz':
-        import gzip
-        infile=gzip.open(options.in_diagnostic_pointers_file,'r')
-    else:
-        infile=open(options.in_diagnostic_pointers_file,'r')
-    paths_dict=json.load(infile)
 
+    new_paths_dict=copy.deepcopy(paths_dict)
     #Slice the database:
-    paths_dict['data_pointers']=database_utils.slice_data(options,paths_dict['data_pointers'])
-
+    new_paths_dict['data_pointers']=database_utils.slice_data(new_paths_dict['data_pointers'],options)
 
     #Remove the simulations that were excluded:
-    simulation_desc=['center','model','rip']
-    intersection_simulations=set(simulation_desc).intersection(dir(options))
-    simulations_list=copy.copy(paths_dict['simulations_list'])
-    if len(intersection_simulations)>0:
-        for simulation in paths_dict['simulations_list']:
-            for desc in intersection_simulations:
-                if simulation.split('_')[simulation_desc.index(desc)]!=getattr(options,desc):
-                    simulations_list.remove(simulation)
-                    break
-    paths_dict['simulations_list']=simulations_list
+    new_paths_dict['simulations_list']=simulations_list(new_paths_dict,options)
+    return new_paths_dict
 
-    outfile=open(options.out_paths_file,'w')
-    json.dump(paths_dict,outfile)
-    return
+def simulations_list(paths_dict,options):
+    import database_utils
+    import copy
+    temp_options=copy.deepcopy(options)
+    simulations_list=[]
+    for center in database_utils.list_level(paths_dict['data_pointers'],temp_options,'center'):
+        setattr(temp_options,'center',center)
+        for model in database_utils.list_level(paths_dict['data_pointers'],temp_options,'model'):
+            setattr(temp_options,'model',model)
+            for rip in database_utils.list_level(paths_dict['data_pointers'],temp_options,'rip'):
+                if rip!='r0i0p0':
+                    simulations_list.append('_'.join([center,model,rip]))
+    return simulations_list
 
 def main():
     import argparse 
@@ -243,10 +260,12 @@ def main():
                                          )
     function_command['optimset']=find_optimset
     optimset_parser.add_argument('in_diagnostic_headers_file',
-                                 type=argparse.FileType('r'),
                                  help='Diagnostic headers file (input)')
     optimset_parser.add_argument('out_paths_file',
                                  help='Diagnostic paths file (output)')
+    optimset_parser.add_argument('-z','--gzip',
+                                 default=False, action='store_true',
+                                 help='Compress the output using gzip. Because the output is mostly repeated text, this leads to large compression.')
     #Find Optimset Months
     epilog_optimset_months=textwrap.dedent(epilog+'\n\nThe output can be pretty printed by using:\n\
                           cat out_paths_file | python -mjson.tool')
@@ -262,7 +281,6 @@ def main():
                                          )
     function_command['optimset_months']=find_optimset_months
     optimset_months_parser.add_argument('in_diagnostic_headers_file',
-                                 type=argparse.FileType('r'),
                                  help='Diagnostic headers file (input)')
     optimset_months_parser.add_argument('out_paths_file',
                                  help='Diagnostic paths file (output)')
@@ -270,30 +288,36 @@ def main():
                                  default=False, action='store_true',
                                  help='Compress the output using gzip. Because the output is mostly repeated text, this leads to large compression.')
 
-    #Retrieve data
-    retrieve_parser=subparsers.add_parser('retrieve',
-                                           help='Retrieve a path (on file system or url) to files containing:\n\
-                                                 a set of variables from a single month, year, model, experiment.\n\
-                                                 It retrieves the latest version.',
+    #Define the data slicing arguments in a dictionnary:
+    slicing_args={
+                  'center': [str,'Modelling center name'],
+                  'model': [str,'Model name'],
+                  'experiment': [str,'Experiment name'],
+                  'rip': [str,'RIP identifier, e.g. r1i1p1'],
+                  'var': [str,'Variable name, e.g. tas'],
+                  'frequency': [str,'Frequency, e.g. day'],
+                  'realm': [str,'Realm, e.g. atmos'],
+                  'mip': [str,'MIP table name, e.g. day'],
+                  'year': [int,'Year'],
+                  'month': [int,'Month as an integer ranging from 1 to 12'],
+                  'file_type': [str,'File type: OPEnDAP, local_file, HTTPServer, GridFTP']
+                  }
+
+    #List_paths
+    list_parser=subparsers.add_parser('list_paths',
+                                           help='List paths (on file system or url) to files containing:\n\
+                                                 ',
                                            )
-    function_command['retrieve']=retrieval
-    retrieve_parser.add_argument('center',type=str,help='Modelling center name')
-    retrieve_parser.add_argument('model',type=str,help='Model name')
-    retrieve_parser.add_argument('experiment',type=str,help='Experiment name')
-    retrieve_parser.add_argument('rip',type=str,help='RIP identifier, e.g. r1i1p1')
-    retrieve_parser.add_argument('var',type=str,help='Variable name, e.g. tas')
-    retrieve_parser.add_argument('frequency',type=str,help='Frequency, e.g. day')
-    retrieve_parser.add_argument('realm',type=str,help='Realm, e.g. atmos')
-    retrieve_parser.add_argument('mip',type=str,help='MIP table name, e.g. day')
-    retrieve_parser.add_argument('year',type=int,help='Year')
-    retrieve_parser.add_argument('month',type=int,help='Month as an integer ranging from 1 to 12')
-    retrieve_parser.add_argument('in_diagnostic_pointers_file',
+    function_command['list_paths']=list_paths
+    for arg in slicing_args.keys():
+        list_parser.add_argument('--'+arg,type=slicing_args[arg][0],help=slicing_args[arg][1])
+    list_parser.add_argument('in_diagnostic_headers_file',
                                  help='Diagnostic headers file with data pointers (input)')
 
-    retrieve_parser.add_argument('-f','--file_type_flag',
+    list_parser.add_argument('-w','--wget',
                                  default=False,
                                  action='store_true',
-                                 help='prints only the file_type if selected')
+                                 help='Prints the paths with a structure that can be passed to a wget script, with MD5 checksum.')
 
     #Slice data
     slice_parser=subparsers.add_parser('slice',
@@ -301,40 +325,50 @@ def main():
                                            argument_default=argparse.SUPPRESS
                                            )
     function_command['slice']=slice_data
-    slice_parser.add_argument('--center',type=str,help='Modelling center name')
-    slice_parser.add_argument('--model',type=str,help='Model name')
-    slice_parser.add_argument('--experiment',type=str,help='Experiment name')
-    slice_parser.add_argument('--rip',type=str,help='RIP identifier, e.g. r1i1p1')
-    slice_parser.add_argument('--var',type=str,help='Variable name, e.g. tas')
-    slice_parser.add_argument('--frequency',type=str,help='Frequency, e.g. day')
-    slice_parser.add_argument('--realm',type=str,help='Realm, e.g. atmos')
-    slice_parser.add_argument('--mip',type=str,help='MIP table name, e.g. day')
-    slice_parser.add_argument('--year',type=int,help='Year')
-    slice_parser.add_argument('--month',type=int,help='Month as an integer ranging from 1 to 12')
-    slice_parser.add_argument('in_diagnostic_pointers_file',
+    for arg in slicing_args.keys():
+        slice_parser.add_argument('--'+arg,type=slicing_args[arg][0],help=slicing_args[arg][1])
+    slice_parser.add_argument('in_diagnostic_headers_file',
                                  help='Diagnostic headers file with data pointers (input)')
     slice_parser.add_argument('out_paths_file',
                                  help='Diagnostic paths file (output)')
+    slice_parser.add_argument('-z','--gzip',
+                                 default=False, action='store_true',
+                                 help='Compress the output using gzip. Because the output is mostly repeated text, this leads to large compression.')
+
+    #find_local
+    find_local_parser=subparsers.add_parser('find_local',
+                                           help='Find the local files that were downloaded'
+                                           )
+    function_command['find_local']=find_local
+    for arg in slicing_args.keys():
+        find_local_parser.add_argument('--'+arg,type=slicing_args[arg][0],help=slicing_args[arg][1])
+    find_local_parser.add_argument('in_diagnostic_headers_file',
+                                 help='Diagnostic headers file with data pointers (input)')
+    find_local_parser.add_argument('out_paths_file',
+                                 help='Diagnostic paths file (output)')
+    find_local_parser.add_argument('-z','--gzip',
+                                 default=False, action='store_true',
+                                 help='Compress the output using gzip. Because the output is mostly repeated text, this leads to large compression.')
 
-    slice_parser.add_argument('--remove',
-                                 default=False,
-                                 action='store_true',
-                                 help='Deletes only the indentified data instead of keeping only the identified data.')
 
     #Simulations
     simulations_parser=subparsers.add_parser('simulations',
                                            help='Prints the (center_model_rip) triples available in the pointers file.'
                                            )
     function_command['simulations']=simulations
-    simulations_parser.add_argument('in_diagnostic_pointers_file',
+    simulations_parser.add_argument('in_diagnostic_headers_file',
                                  help='Diagnostic headers file with data pointers (input)')
 
+
     options=parser.parse_args()
 
+    #Load pointer file:
+    paths_dict=open_json(options)
     #Run the command:
-    function_command[options.command](options)
+    paths_dict=function_command[options.command](paths_dict,options)
+    #Close the file:
+    if not paths_dict==None:
+        close_json(paths_dict,options)
 
 if __name__ == "__main__":
-    #import cProfile
-    #cProfile.run('main()','optimset.prof')
     main()
