@@ -1,4 +1,5 @@
 import copy
+import os
 import database_utils
 from database_utils import File_Expt
 
@@ -141,6 +142,11 @@ def intersection(paths_dict,diag_tree_desc, diag_tree_desc_final):
     #          for all months of all years for all experiments.
     model_list=[tuple(simulation.split('_')) for simulation in paths_dict['simulations_list']]
 
+    if 'months_list' in paths_dict['diagnostic'].keys():
+        diag_months_list=paths_dict['diagnostic']['months_list']
+    else:
+        diag_months_list=range(1,13)
+
     for experiment in paths_dict['diagnostic']['experiment_list'].keys():
         period_list = paths_dict['diagnostic']['experiment_list'][experiment]
         if not isinstance(period_list,list): period_list=[period_list]
@@ -151,7 +157,7 @@ def intersection(paths_dict,diag_tree_desc, diag_tree_desc_final):
             years_list.append(years_range[1])
         months_list=[]
         for year in years_list:
-            for month in range(1,13):
+            for month in diag_months_list:
                 months_list.append((year,month))
 
         model_list_var=copy.copy(model_list)
@@ -177,14 +183,14 @@ def intersection(paths_dict,diag_tree_desc, diag_tree_desc_final):
                             if not set(months_list).issubset(months_list_var):
                                 missing_vars.append(var_name+':'+','.join(
                                                     paths_dict['diagnostic']['variable_list'][var_name])+
-                                                    ' for some months in years:'+','.join(
+                                                    ' for some months in years: '+','.join(
                                                     sorted(set(str(year) for (year,month) in set(months_list).difference(months_list_var)))
                                                     )
                                                    )
             if len(missing_vars)>0:
-               print('_'.join(model)),
-               print(', experiment '+experiment+' is missing '),
-               print(missing_vars)
+               #print('\nThe reasons why some simulations were excluded:')
+               print('_'.join(model)+' excluded because experiment '+experiment+' is missing variables:\n'),
+               for item in missing_vars: print(item)
                model_list.remove(model)
 
     #Step two: create the new paths dictionary:
@@ -225,12 +231,12 @@ def intersection(paths_dict,diag_tree_desc, diag_tree_desc_final):
                             #Works if the variable is not fx:
                             if item.rip==model[2]:
                                 if int(item.year) in years_list:
-                                    if int(item.month) in range(1,13):
+                                    if int(item.month) in diag_months_list:
                                         new_paths_dict['data_pointers']=database_utils.create_tree(item,diag_tree_desc_final,new_paths_dict['data_pointers'])
                         else:
                             #If fx, we create the time axis for easy retrieval:
                             for year in years_list:
-                                for month in range(1,13):
+                                for month in diag_months_list:
                                     item.year=year
                                     item.month=month
                                     new_paths_dict['data_pointers']=database_utils.create_tree(item,diag_tree_desc_final,new_paths_dict['data_pointers'])
